/*---------------------------------------------------------------------------*/
/*                                PRG-2051                                   */
/*                                                                           */
/* Device programmer for the Atmel 89C2051 and 89C1051 microcontrollers.     */
/*                                                                           */
/* Filename:    COMMAND.C                                                    */
/* Description: User interface command functions.                            */
/*                                                                           */
/* Copyright (c) 1995,1996,1997 Electronic Design Associates.                */
/* All rights reserved.                                                      */
/*                                                                           */
/*---------------------------------------------------------------------------*/
/*                                                                           */
/* Ver 1.1 01/19/97	Added verify() to program().                             */
/*                  Added code to reset program counter in program().        */
/*                                                                           */
/*---------------------------------------------------------------------------*/

#include <prg2051.h>
#include <hardware.h>
#include <reg52.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <uart.h>
#include <bios.h>
#include <subrs.h>

xdata char prog_buffer[BUFFER_SIZE];
char device=ATMEL_89C2051;
const int max_address[]={ 0, 1024, 2048 };


/*---------------------------------------------------------------------------*/
/*                                blank()                                    */
/*---------------------------------------------------------------------------*/
void blankcheck(void)
{
int address;

  device_on();

  printf("\n\rBlank checking...");

  set_cmd(READ_DATA);
  for (address=0; address<max_address[device]; address++) {
    if ( get_data() != 0xff ) {
      printf("\rBlank check failed at address %04X.", address);
      device_off();
      return;
    }
    clock_on();
    clock_off();
  }

  printf("\rDevice is blank.                ");

  device_off();
}


/*---------------------------------------------------------------------------*/
/*                               checksum()                                  */
/*---------------------------------------------------------------------------*/
void checksum(void)
{
unsigned int sum=0;
unsigned int address=0;

  printf("\n\rCalculating checksum...");
  for (address=0; address<BUFFER_SIZE; address++)
    sum+=prog_buffer[address];
  printf("\rChecksum = %04X            ",sum);
}


/*---------------------------------------------------------------------------*/
/*                                 dump()                                    */
/*---------------------------------------------------------------------------*/
void dump(int dump_address)
{
static int address=0;
char line_num,index;

  if (dump_address>=0) address=dump_address;
  if (address>=BUFFER_SIZE) address=0;

  printf("\n\r");

  for (line_num=8; line_num; line_num--) {

    printf("\r\n\r%04X  ",address);

    for (index=0; index<16; index++) {
      printf("%02bX ",prog_buffer[address]);
      if (index==7) printf("- ");
      address++;
    }

    printf("  ");

    for (index=16; index; index--) {
      if (isgraph(prog_buffer[address-index]))
        putchar(prog_buffer[address-index]);
      else
        putchar('.');
    }

  }
}


/*---------------------------------------------------------------------------*/
/*                               enter()                                     */
/*---------------------------------------------------------------------------*/
void enter(char *cptr)
{
int address;
char byte;

  address=strtoi(cptr);
  while (isspace(*cptr)) cptr++;
  while (isxdigit(*cptr)) cptr++;
  if (*cptr==NULL) return;

  for(;;) {
    while (isspace(*cptr)) cptr++;
    if (!isxdigit(*cptr)) return;
    byte=(char)strtoi(cptr);
    prog_buffer[address]=byte;
    address++;
    while (isspace(*cptr)) cptr++;
    while (isxdigit(*cptr)) cptr++;
  }
}


/*---------------------------------------------------------------------------*/
/*                               erase()                                     */
/*---------------------------------------------------------------------------*/
void erase(void)
{
  device_on();

  printf("\n\rErasing...");
  set_cmd(CHIP_ERASE);

  vpp_on();
  delay10us();
  prog_off();
  delay10ms();
  prog_on();
  delay10us();
  vpp_off();

  printf("\rDevice erased.");

  device_off();
}


/*---------------------------------------------------------------------------*/
/*                                fill()                                     */
/*---------------------------------------------------------------------------*/
void fill(char *cptr)
{
int address,start,stop;
char byte;

  start=strtoi(cptr);
  while (isspace(*cptr)) cptr++;
  while (isxdigit(*cptr)) cptr++;
  if (*cptr==NULL) return;

  stop=strtoi(cptr);
  while (isspace(*cptr)) cptr++;
  while (isxdigit(*cptr)) cptr++;
  if (*cptr==NULL) return;

  byte=(char)strtoi(cptr);

  if (start>stop) return;
  if (stop>=BUFFER_SIZE) return;


  for (address=start; address<=stop; address++) {
    prog_buffer[address]=byte;
  }

  printf("\n\rDone.");
}


/*---------------------------------------------------------------------------*/
/*                                help()                                     */
/*---------------------------------------------------------------------------*/
void help(void)
{
  printf("\n\rAuto      A               Erase, program, verify, & lock device    ");
  printf("\n\rBlank     B               Blank check device                       ");
  printf("\n\rChecksum  C               Calculate 16-bit checksum on memory      ");
  printf("\n\rDump      D [address]     Dump memory contents                     ");
  printf("\n\rErase     E               Erase device                             ");
  printf("\n\rFill      F range data    Fill memory                              ");
  printf("\n\rlocK      K               Program device lock bits                 ");
  printf("\n\rLoad      L               Load Intel hex file to memory            ");
  printf("\n\reNter     N address list  Enter data into memory                   ");
  printf("\n\rProgram   P               Erase & Program device                   ");
  printf("\n\rRead      R               Read from device to memory               ");
  printf("\n\rSignature S               Display device signature bytes           ");
  printf("\n\rType      T [device]      Display or Set device type               ");
  printf("\n\r                            device = 1 for 89C1051                 ");
  printf("\n\r                            device = 2 for 89C2051 (default)       ");
  printf("\n\r                            displays current type if not specified ");
  printf("\n\rVerify    V               Verify device against memory             ");
}


/*---------------------------------------------------------------------------*/
/*                                load()                                     */
/*---------------------------------------------------------------------------*/
void load(void)
{
unsigned int address;
unsigned char byte_count;
unsigned char record_type;
unsigned char check_sum;
unsigned char ctemp;

  printf("\n\r\n\rBegin file transmission now.\n\r");

  for (;;) {

    for (;;) {
      if ( !(ctemp = getch()) ) return;
      if (ctemp == ':') break;
    }

    if ( !(Hex[1] = getch()) ) return;
    if ( !(Hex[0] = getch()) ) return;
    ctemp = hex2bin();
    check_sum = ctemp;
    byte_count = ctemp;

    if ( !(Hex[1] = getch()) ) return;
    if ( !(Hex[0] = getch()) ) return;
    ctemp = hex2bin();
    check_sum += ctemp;
    address = ctemp;
    address *= 256;

    if ( !(Hex[1] = getch()) ) return;
    if ( !(Hex[0] = getch()) ) return;
    ctemp = hex2bin();
    check_sum += ctemp;
    address += ctemp;

    if ( !(Hex[1] = getch()) ) return;
    if ( !(Hex[0] = getch()) ) return;
    ctemp = hex2bin();
    check_sum += ctemp;
    record_type = ctemp;

    while (byte_count) {
      if ( !(Hex[1] = getch()) ) return;
      if ( !(Hex[0] = getch()) ) return;
      ctemp = hex2bin();
      check_sum += ctemp;
      if (address>=BUFFER_SIZE) {
        printf("\n\rAddress out of range.");
        return;
      }
      prog_buffer[address] = ctemp;
      address++;
      byte_count--;
    }

    if ( !(Hex[1] = getch()) ) return;
    if ( !(Hex[0] = getch()) ) return;
    ctemp = hex2bin();

    if ( ctemp + check_sum ) {
      printf("\n\rBad checksum");
      return;
    }

    putchar('.');

    if (record_type==END_RECORD) break;
  }

  printf("\n\rDone");
}


/*---------------------------------------------------------------------------*/
/*                                lock()                                     */
/*---------------------------------------------------------------------------*/
void lock(void)
{
  device_on();

  printf("\n\rLocking...");

  set_cmd(LOCK_BIT1);
  vpp_on();
  delay10us();
  prog_off();
  delay10us();
  prog_on();
  delay10ms();

  set_cmd(LOCK_BIT2);
  delay10us();
  prog_off();
  delay10us();
  prog_on();
  delay10us();
  vpp_off();

  printf("\rDevice locked.");

  device_off();
}


/*---------------------------------------------------------------------------*/
/*                              verify()                                     */
/*---------------------------------------------------------------------------*/
char verify(void)
{
int address;
char byte;

  device_on();

  printf("\n\rVerifying...");

  set_cmd(READ_DATA);
  for (address=0; address<max_address[device]; address++) {
    if ( prog_buffer[address] != (byte=get_data()) ) {
      printf("\rVerify failed at address %04X.",address);
      printf("\n\rDevice data = %02bX",byte);
      printf("\n\rMemory data = %02bX",prog_buffer[address]);
      device_off();
      return(0);
    }
    clock_on();
    clock_off();
  }

  printf("\rDevice verifies.");

  device_off();
  return(1);
}


/*---------------------------------------------------------------------------*/
/*                                prog()                                     */
/*---------------------------------------------------------------------------*/
void prog(void)
{
int address;
unsigned char cnt=0;

  device_on();

  printf("\n\rErasing...");
  set_cmd(CHIP_ERASE);

  vpp_on();
  delay10us();
  prog_off();
  delay10ms();
  prog_on();
  delay10us();
  vpp_off();

  printf("\rProgramming....");

  reset_off();	/* Ver 1.1 1/19/97 - set programming counter to zero */
  delay10ms();
  reset_on();

  for (address=0; address<BUFFER_SIZE; address++) {
    if ( cnt++ >= 40) cnt=0;
    switch(cnt) {
      case 0:
        putchar('\b');
        putchar('|');
        break;
      case 10:
        putchar('\b');
        putchar('/');
        break;
      case 20:
        putchar('\b');
        putchar('-');
        break;
      case 30:
        putchar('\b');
        putchar('\\');
        break;
    }
    if ( prog_byte(prog_buffer[address]) ) {
      printf("\n\rProgramming failed at address %04x.",address);
      device_off();
      return;
    }
    clock_on();
    clock_off();
  }

  printf("\rDevice programmed.");

  device_off();
}


/*---------------------------------------------------------------------------*/
/*                               read()                                      */
/*---------------------------------------------------------------------------*/
void read(void)
{
int address;

  device_on();

  printf("\n\rReading...");

  set_cmd(READ_DATA);
  for (address=0; address<max_address[device]; address++) {
    prog_buffer[address] = get_data();
    clock_on();
    clock_off();
  }

  printf("\rDone.              ");

  device_off();
}


/*---------------------------------------------------------------------------*/
/*                             signature()                                   */
/*---------------------------------------------------------------------------*/
void signature(void)
{
  device_on();

  set_cmd(READ_SIGN);
  printf("\n\rByte0 = %02bX", get_data() );
  
  clock_on();
  clock_off();

  printf("\n\rByte1 = %02bX", get_data() );
  
  clock_on();
  clock_off();
  
  printf("\n\rByte2 = %02bX", get_data() );

  device_off();
}


/*---------------------------------------------------------------------------*/
/*                               type()                                      */
/*---------------------------------------------------------------------------*/
void type(char new_device)
{
static char device_strings[][20] = { "None", "89C1051", "89C2051" };

  if (new_device) device=new_device;
  printf("\n\rDevice Type = %s", device_strings[device]);
}


/*---------------------------------------------------------------------------*/
/*                                info()                                     */
/*---------------------------------------------------------------------------*/
void info(void)
{
  printf("\n\rModel: PRG-2051 ");
  printf("\n\r Date: 11/12/95 ");
  printf("\n\r  Ver: 1.0      ");
}


/*---------------------------------------------------------------------------*/
/*                              program()                                    */
/*---------------------------------------------------------------------------*/
void program(void)
{
  prog();
  verify();
}


/*---------------------------------------------------------------------------*/
/*                                auto()                                     */
/*---------------------------------------------------------------------------*/
void automatic(void)
{
  prog();
  if ( verify() ) lock();
}


